#######################################################################
# cumulative cross-section script - author Kai Urban - last edit 25.02.2022
#######################################################################

# load necessary packages
import numpy as np
from scipy import integrate

import DMgamSpec.consts
from DMgamSpec.zerobins import ZBwino, ZBhig
from DMgamSpec.diffcross import diffwino, diffhig

#######################################################################
# Cumulate cross-sections from the endpoint
# (output dimension 10^(-26) cm^3/s)
# function arguments - x [], mass [TeV], model (string), SF (string), ZBsize
# (string - optional)
#######################################################################

# helper function for zero-bin size determination
def zerobinsize(ZBsize):
    if ZBsize == 'default':
        return DMgamSpec.consts.ZB_oneminx
    elif ZBsize == '1 %':
        return DMgamSpec.consts.ZB_onepercent
    else: 
        print('!!! Error - zero-bin size not supported - please choose a valid zero-bin !!!')
        return 'error'

def cumulwino(x, mchi, SF, ZBsize = 'default', rel = -3):
    # check for zero-bin size
    ZBvalue = zerobinsize(ZBsize)
    if ZBvalue == 'error':
        return None

    # if x inside zero-bin return zero-bin value, other wise integrate from
    # zero-bin value to x and add zero-bin on top
    if 1-x <= ZBvalue:
        intdiff = np.array([0])
        print('Value of x within the zerobin, 1-x set to ' + str(ZBvalue))
    else:
        # make integrator aware of points where sharp features are located
        # Z-pole, WWgamma threshold and 0.01 to split integral appropriately
        # note, if point is not in integration range, it will be ignored
        ptlist = [1-0.0911876**2/(4*mchi**2), 1-0.080385**2/mchi**2, 1-0.01]
        intdiff = integrate.quad(diffwino, x, 1 - ZBvalue, limit=250, \
                epsrel=10**(rel), epsabs = 0, points = ptlist, args=(mchi,SF,))
    return ZBwino(mchi, SF, ZBsize) + intdiff[0]

def cumulhig(x, mchi, SF, ZBsize = 'default', rel = -3):
    # determine zero-bin size
    ZBvalue = zerobinsize(ZBsize)
    if ZBvalue == 'error':
        return None

    # if x inside zero-bin return zero-bin value, other wise integrate from
    # zero-bin value to x and add zero-bin on top
    if 1-x <= ZBvalue:
        intdiff = np.array([0])
        print('Value of x within the zerobin, 1-x set to ' + str(ZBvalue))
    else:
        # make integrator aware of points where sharp features are located
        # Z-pole, WWgamma threshold and 0.01 to split integral appropriately
        # note, if point is not in integration range, it will be ignored
        ptlist = [1-0.0911876**2/(4*mchi**2), 1-0.080385**2/mchi**2, 1-0.01]
        intdiff = integrate.quad(diffhig, x, 1 - ZBvalue, limit=250, \
                epsrel=10**(rel), epsabs = 0, points = ptlist, args=(mchi,SF,))
    return ZBhig(mchi, SF, ZBsize) + intdiff[0]
