#######################################################################
# diff. cross-section script - author Kai Urban - last edit 04.02.2022
#######################################################################

# load necessary packages
import numpy as np
from scipy import interpolate 

import DMgamSpec.consts
from DMgamSpec.load_SF import winoSFactor, higgsinoSFactor

#######################################################################
# Load the wino grid and parse to interpolating functions for Gamma_IJ
#######################################################################

winogrid = np.genfromtxt('./data/WinoGridGIJ.csv', delimiter = ',') # load wino grid

dimwino = np.genfromtxt('./data/dimensionsWinoGrid.csv', delimiter=',', dtype=int).flatten() # load dim. of grid

winogrid = winogrid.reshape([dimwino[0],dimwino[1],dimwino[2]]) # reshape arrays for interpolation

# interpolation of the various GammaIJ elements 00 00, 00 pm (for real and
# imaginary part) and the (+-)(+-) element, units 1/TeV**2 conversion for
# final cross-section functions
wino_00 = interpolate.interp2d(winogrid[0,:,1], winogrid[:,0,0], winogrid[:,:,2], kind='linear')
wino_0pre = interpolate.interp2d(winogrid[0,:,1], winogrid[:,0,0], winogrid[:,:,3], kind='linear')
wino_0pim = interpolate.interp2d(winogrid[0,:,1], winogrid[:,0,0], winogrid[:,:,4], kind='linear')
wino_pm = interpolate.interp2d(winogrid[0,:,1], winogrid[:,0,0], winogrid[:,:,5], kind='linear')

#######################################################################
# Load the higgsino grid and parse to interpolating functions for Gamma_IJ
#######################################################################

higgsinogrid = np.genfromtxt('./data/HiggsinoGridGIJ.csv', delimiter = ',') # load higgsino grid

dimhiggsino = np.genfromtxt('./data/dimensionsHiggsinoGrid.csv', delimiter=',', dtype=int).flatten() # load dim. of grid

higgsinogrid = higgsinogrid.reshape([dimhiggsino[0], dimhiggsino[1], dimhiggsino[2]]) # reshape arrays for interpolation

# interpolation of the various GammaIJ elements 00 00, 00 pm (for real and
# imaginary part) and the (+-)(+-) element, units 1/TeV**2 conversion for
# final cross-section functions, note that 11 and 22 elements only differ in
# the Sommerfeld factor
hig_00 = interpolate.interp2d(higgsinogrid[0,:,1], higgsinogrid[:,0,0], higgsinogrid[:,:,2], kind='linear')
hig_0pre = interpolate.interp2d(higgsinogrid[0,:,1], higgsinogrid[:,0,0], higgsinogrid[:,:,3], kind='linear')
hig_0pim = interpolate.interp2d(higgsinogrid[0,:,1], higgsinogrid[:,0,0], higgsinogrid[:,:,4], kind='linear')
hig_pm = interpolate.interp2d(higgsinogrid[0,:,1], higgsinogrid[:,0,0], higgsinogrid[:,:,5], kind='linear')

#######################################################################
# Differential cross-sections 
# (output dimension 10^(-26) cm^3/s)
# (function arguments - x [], mass [TeV], model (string), SF (string))
#######################################################################

def diffwino(x, mchi, SF):
    # add in Sommerfeld factors, complex parts via S0p * G0p + S0p^* * G0p^*
    # = 2 Re[S0p] Re[G0p] - 2 Im[S0p] Im[G0p]
    
    Sfactor = winoSFactor(mchi, SF)

    Xsection = Sfactor[0] * wino_00(1-x, mchi) \
            + 2 * Sfactor[1] * wino_0pre(1-x, mchi) \
            - 2 * Sfactor[2] * wino_0pim(1-x, mchi) \
            + Sfactor[3] * wino_pm(1-x, mchi)
            
    # factor 2 for method II Sommerfeld factors
    return 2 * Xsection[0] * DMgamSpec.consts.TeV2tocm3s 

def diffhig(x, mchi, SF):
    # add in Sommerfeld complex parts with e.g., S0p * G0p + S0p^* * G0p^*
    # = 2 Re[S0p] Re[G0p] - 2 Im[S0p] Im[G0p]
    # order of Sommerfeld arguments is
    # S_1111, Re[S_1122], Im[S_1122], Re[S_11+-], Im[S_11+-], 
    # S_2222, Re[S_22+-], Im[S_22+-], S_+-+-
    Sfactor = higgsinoSFactor(mchi, SF)
    
    Xsection = (Sfactor[0] + 2 * Sfactor[1] + Sfactor[2] - Sfactor[2] + Sfactor[5]) * hig_00(1-x, mchi) \
            + (2 * Sfactor[3] + 2 * Sfactor[6]) * hig_0pre(1-x, mchi) \
            - (2 * Sfactor[4] + 2 * Sfactor[7]) * hig_0pim(1-x, mchi) \
            + Sfactor[8] * hig_pm(1-x, mchi)
            
    # factor 2 for method II Sommerfeld factors
    return 2 * Xsection[0] * DMgamSpec.consts.TeV2tocm3s 
