#######################################################################
# load GIJ grids script - author Kai Urban - last edit 04.02.2022
#######################################################################

import numpy as np
from scipy import interpolate 

import DMgamSpec.consts
from DMgamSpec.load_SF import winoSFactor, higgsinoSFactor

#######################################################################
# Load the zero-bin wino grid and parse to interpolating functions Gamma_IJ
#######################################################################

# there are two zero-bin grid options offered, default is 1-x = 2 * 10^(-5)
# if explicitly the flag is put into functions also 1-x = 0.01 can be used to
# circumvent possible interpolation and integration errors
ZB_wino_grid_default = np.genfromtxt('./data/ZBGrid_wino_GIJ_2-5.csv', delimiter = ',') # load default 1-x = 2 * 10^(-5) zerobin grid
ZB_wino_grid_onepercent = np.genfromtxt('./data/ZBGrid_wino_GIJ_1-2.csv', delimiter = ',') # load 1-x = 0.01 zerobin grid

# interpolation of the analogue to the above for the default zero-bin 
# (1D, as the zero bin width is fixed to 0 to 2*10**(-5) in 1-x)
wino_ZB_00_default = interpolate.interp1d(ZB_wino_grid_default[:,0], ZB_wino_grid_default[:,1], kind='linear')
wino_ZB_0pre_default = interpolate.interp1d(ZB_wino_grid_default[:,0], ZB_wino_grid_default[:,2], kind='linear')
wino_ZB_0pim_default = interpolate.interp1d(ZB_wino_grid_default[:,0], ZB_wino_grid_default[:,3], kind='linear')
wino_ZB_pm_default = interpolate.interp1d(ZB_wino_grid_default[:,0], ZB_wino_grid_default[:,4], kind='linear')

# interpolation of the one percent zero-bin (as above)
# (1D, as the zero bin width is fixed to 0 to 1*10**(-2) in 1-x)
wino_ZB_00_onepercent = interpolate.interp1d(ZB_wino_grid_onepercent[:,0], ZB_wino_grid_onepercent[:,1], kind='linear')
wino_ZB_0pre_onepercent = interpolate.interp1d(ZB_wino_grid_onepercent[:,0], ZB_wino_grid_onepercent[:,2], kind='linear')
wino_ZB_0pim_onepercent = interpolate.interp1d(ZB_wino_grid_onepercent[:,0], ZB_wino_grid_onepercent[:,3], kind='linear')
wino_ZB_pm_onepercent = interpolate.interp1d(ZB_wino_grid_onepercent[:,0], ZB_wino_grid_onepercent[:,4], kind='linear')

#######################################################################
# Load the zero-bin higgsino grid and parse to interpolating functions Gamma_IJ
#######################################################################

# there are two zero-bin grid options offered, default is 1-x = 2 * 10^(-5)
# if explicitly the flag is put into functions also 1-x = 0.01 can be used to
# circumvent possible interpolation and integration errors
ZB_hig_grid_default = np.genfromtxt('./data/ZBGrid_higgsino_GIJ_2-5.csv', delimiter = ',') # load default 1-x = 2 * 10^(-5) zerobin grid
ZB_hig_grid_onepercent = np.genfromtxt('./data/ZBGrid_higgsino_GIJ_1-2.csv', delimiter = ',') # load 1-x = 0.01 zerobin grid

# interpolation of the analogue to the above for the zero-bin 
# (1D, as the zero bin width is fixed to 0 to 2*10**(-5) in 1-x)
hig_ZB_00_default = interpolate.interp1d(ZB_hig_grid_default[:,0], ZB_hig_grid_default[:,1], kind='linear')
hig_ZB_0pre_default = interpolate.interp1d(ZB_hig_grid_default[:,0], ZB_hig_grid_default[:,2], kind='linear')
hig_ZB_0pim_default = interpolate.interp1d(ZB_hig_grid_default[:,0], ZB_hig_grid_default[:,3], kind='linear')
hig_ZB_pm_default = interpolate.interp1d(ZB_hig_grid_default[:,0], ZB_hig_grid_default[:,4], kind='linear')

# interpolation of the one percent zero-bin (as above)
# (1D, as the zero bin width is fixed to 0 to 1*10**(-2) in 1-x)
hig_ZB_00_onepercent = interpolate.interp1d(ZB_hig_grid_onepercent[:,0], ZB_hig_grid_onepercent[:,1], kind='linear')
hig_ZB_0pre_onepercent = interpolate.interp1d(ZB_hig_grid_onepercent[:,0], ZB_hig_grid_onepercent[:,2], kind='linear')
hig_ZB_0pim_onepercent = interpolate.interp1d(ZB_hig_grid_onepercent[:,0], ZB_hig_grid_onepercent[:,3], kind='linear')
hig_ZB_pm_onepercent = interpolate.interp1d(ZB_hig_grid_onepercent[:,0], ZB_hig_grid_onepercent[:,4], kind='linear')

#######################################################################
# Zero-bin function - necessary for cumulative cross-sections
# common zero-bin width given in variable ZB_oneminx or ZB_onepercent
# (output dimension 10^(-26) cm^3/s)
# (function arguments - mass [TeV], SF (string), ZBsize (string - optional)
#######################################################################

def ZBwino(mchi, SF, ZBsize = 'default'):
    Sfactor = winoSFactor(mchi, SF)

    # choose the appropriate zero-bin size
    if ZBsize == 'default':
        ZB00 = wino_ZB_00_default(mchi)
        ZB0pre = wino_ZB_0pre_default(mchi)
        ZB0pim = wino_ZB_0pim_default(mchi)
        ZBpm = wino_ZB_pm_default(mchi)
    elif ZBsize == '1 %':
        ZB00 = wino_ZB_00_onepercent(mchi)
        ZB0pre = wino_ZB_0pre_onepercent(mchi)
        ZB0pim = wino_ZB_0pim_onepercent(mchi)
        ZBpm = wino_ZB_pm_onepercent(mchi)
    else:
        return print('!!! Error - zero-bin size not supported - please choose a valid zero-bin !!!')

    # add in Sommerfeld factors, complex parts via S0p * G0p + S0p^* * G0p^*
    # = 2 Re[S0p] Re[G0p] - 2 Im[S0p] Im[G0p]
    zerobinint = Sfactor[0] * ZB00 + 2 * Sfactor[1] * ZB0pre \
            - 2 * Sfactor[2] * ZB0pim + Sfactor[3] * ZBpm
            
    # factor 2 for method II Sommerfeld factors
    return 2 * zerobinint * DMgamSpec.consts.TeV2tocm3s 

def ZBhig(mchi, SF, ZBsize = 'default'):
    Sfactor = higgsinoSFactor(mchi, SF)

    # choose the appropriate zero-bin size
    if ZBsize == 'default':
        ZB00 = hig_ZB_00_default(mchi)
        ZB0pre = hig_ZB_0pre_default(mchi)
        ZB0pim = hig_ZB_0pim_default(mchi)
        ZBpm = hig_ZB_pm_default(mchi)
    elif ZBsize == '1 %':
        ZB00 = hig_ZB_00_onepercent(mchi)
        ZB0pre = hig_ZB_0pre_onepercent(mchi)
        ZB0pim = hig_ZB_0pim_onepercent(mchi)
        ZBpm = hig_ZB_pm_onepercent(mchi)
    else:
        return print('!!! Error - zero-bin size not supported - please choose a valid zero-bin !!!')

    # add in Sommerfeld complex parts with e.g., S0p * G0p + S0p^* * G0p^*
    # = 2 Re[S0p] Re[G0p] - 2 Im[S0p] Im[G0p]
    # order of Sommerfeld arguments is
    # S_1111, Re[S_1122], Im[S_1122], Re[S_11+-], Im[S_11+-], 
    # S_2222, Re[S_22+-], Im[S_22+-], S_+-+-
    zerobinint = (Sfactor[0] + 2 * Sfactor[1] + Sfactor[2] - Sfactor[2] + Sfactor[5]) * ZB00 \
            + (2 * Sfactor[3] + 2 * Sfactor[6]) * ZB0pre \
            - (2 * Sfactor[4] + 2 * Sfactor[7]) * ZB0pim \
            + Sfactor[8] * ZBpm
            
    # factor 2 for method II Sommerfeld factors
    return 2 * zerobinint * DMgamSpec.consts.TeV2tocm3s 
