#######################################################################
# top-level functions script - author Kai Urban - last edit 24.02.2022
#######################################################################

# load necessary packages
from DMgamSpec.zerobins import ZBwino, ZBhig
from DMgamSpec.diffcross import diffwino, diffhig
from DMgamSpec.cumulcross import zerobinsize, cumulwino, cumulhig
from DMgamSpec.binnedcross import binnedwino, binnedhig

def diffxsection(x, mchi, model, SF):
    
    '''Differential cross-section [10^(-26) cm^3/s]
    
    Parameters
    ----------
    x : float
        Egamma / mchi (restricted between 0 and 1).
    mchi : float
        DM mass in TeV between 0.5 to 100 TeV.
    model : string
        specify model, possible values 'wino' and 'higgsino'.
    SF : string
        specify Sommerfeld tables -- possible values, see Tables 1 and 2 in 
        the documentation.

    Returns
    -------
    numpy.float64
        Differential cross-section at x for model and Sommerfeld factor. 
        Units [10^(-26) * cm^3/s].

    '''
    
    if model == 'wino':
        return diffwino(x, mchi, SF)
    elif model == 'higgsino':
        return diffhig(x, mchi, SF)
    else:
        return print('!!! Error - desired model not implemented - possible models wino or higgsino !!!')

def zerobin(mchi, model, SF, ZBsize = 'default'):
    
    '''Zeroth bin [10^(-26) cm^3/s]
    
    Parameters
    ----------
    mchi : float
        DM mass in TeV (possible values between .5 and 100 TeV).
    model : string
        specify model, possible values 'wino' and 'higgsino'.
    SF : string
        specify Sommerfeld tables -- possible values, see Tables 1 and 2 in 
        the documentation.
    ZBsize : string, optional
        specify zero-bin size, possible values 'default' [1,1-2*10^(-5)] and
        '1 %' [1, 1-0.01]. The default is 'default'.

    Returns
    -------
    numpy.float64
        zeroth bin, integrated from endpoint x = 1 to size of zero-bin defined 
        in ZBsize. Units [-10^(26) * cm^3/s].

    '''
    
    if model == 'wino':
        return ZBwino(mchi, SF, ZBsize)
    elif model == 'higgsino':
        return ZBhig(mchi, SF, ZBsize)
    else:
        return print('!!! Error - desired model not implemented - possible models wino or higgsino !!!')


def cumulxsection(x, mchi, model, SF, ZBsize = 'default', rel = -3):
    
    '''Cross-section cumulated from the endpoint [10^(-26) cm^3/s]
    
    Parameters
    ----------
    x : float
        Egamma / mchi (restricted between 0 and 1).
    mchi : float
        DM mass in TeV between 0.5 to 100 TeV.
    model : string
        specify model, possible values 'wino' and 'higgsino'.
    SF : string
        specify Sommerfeld tables -- possible values, see Tables 1 and 2 in 
        the documentation.
    ZBsize : string, optional
        specify zero-bin size, possible values 'default' [1,1-2*10^(-5)] and
        '1 %' [1, 1-0.01]. The default is 'default'.
    rel : float or int, optional
        specify the relative error to be used for the numerical integration
        of the differential spectrum. Number refers to a relative error of 
        10^(rel). Default is a relative error of 10^(-3).

    Returns
    -------
    numpy.float64
        Cumulate cross-section from 1 to x for model and Sommerfeld factor. 
        Units [10^(-26) * cm^3/s].

    '''
    
    if model == 'wino':
        return cumulwino(x, mchi, SF, ZBsize, rel)
    elif model == 'higgsino':
        return cumulhig(x, mchi, SF, ZBsize, rel)
    else:
        return print('!!! Error - desired model not implemented - possible models wino or higgsino !!!')


# function checks for plausibility of the bins, and if everything is ok, takes
# wino or higgsino binned cross-section

def binnedxsection(mchi, E1, E2, model, SF, ZBsize = 'default', rel = -3):
    
    '''Cross-section binned between E_2 and E_1 [10^(-26) cm^3/s]
    
    Parameters
    ----------
    mchi : float
        DM mass in TeV between 0.5 to 100 TeV.
    E1 : float
        Egamma lower boundary of energy bin in TeV.
    E2 : float
        Egamma upper boundary of energy bin in TeV.
    model : string
        specify model, possible values 'wino' and 'higgsino'.
    SF : string
        specify Sommerfeld tables -- possible values, see Tables 1 and 2 in 
        the documentation.
    ZBsize : string, optional
        specify zero-bin size, possible values 'default' [1,1-2*10^(-5)] and
        '1 %' [1, 1-0.01]. The default is 'default'.
    rel : float or int, optional
        specify the relative error to be used for the numerical integration
        of the differential spectrum. Number refers to a relative error of 
        10^(rel). Default is a relative error of 10^(-3).

    Returns
    -------
    numpy.float64
        bin cross-section from E1 to E2 for model and Sommerfeld factor. 
        Units [10^(-26) * cm^3/s].

    '''    
    
    # determine zero-bin size
    ZBvalue = zerobinsize(ZBsize)
    if ZBvalue == 'error':
        return None

    # for implemented checks, see error messages
    if E1 > E2:
        print('!! Error first bin limit must be smaller than second bin limit !!')
        return None
    elif E2 > mchi or E1 < 0:
        print('!! Error upper or lower bin limit unphysical !!')
        return None
    elif (mchi - E2) < ZBvalue * mchi:
        print('!! Bin overlaps with smallest possible bin, upper bin limit E2 set to mchi !!')
        if (mchi - E1) < ZBvalue * mchi:
            print('!! Error second bin limit lies inside smallest possible bin. Please choose a larger bin size !!')
            return None

    # if none of the checks fails, output the respective model bin
    if model == 'wino':
        return binnedwino(mchi, E1, E2, SF, ZBsize, rel)
    elif model == 'higgsino':
        return binnedhig(mchi, E1, E2, SF, ZBsize, rel)
    else:
        print('!!! Error - desired model not implemented - possible models wino or higgsino !!!')
        return None
